 	PAGE	64,132
	TITLE	INIBAK - device driver to copy files on startup
	NAME	INIBAK
	.286
;
; Description:
;	OS/2 device driver for copying system files.
; Version:
;	1.0
; Date:
;	8th February 1991
; Author:
;	Bob Eager
;		USENET: rde@ukc.ac.uk
;		CIS:	100016,2770
;
;	This program is released into the public domain. Please do not
;	change this notice, or try to make money from the program.
;
; This device driver performs no useful function except at system
; initialisation time. At that time, it copies the files:
;
;	C:\OS2\OS2.INI      to   C:\OS2\OS2.INB
;	C:\OS2\OS2SYS.INI   to   C:\OS2\OS2SYS.INB
;
; before they are opened by the system. This means that normal disk backups
; will include a consistent copy of the above files.
;
; The names of the files are hard coded into the driver, but are easy to
; change; some space has been left so that the driver can be patched to use
; different names if an assembler isn't handy. All names are zero terminated.
;
; Constants
;
STDOUT		EQU	1		; Standard output handle
DCPY_EXISTING	EQU	1		; DosCopy option
;
CR		EQU	0DH		; Carriage return
LF		EQU	0AH		; Linefeed
;
; External references
;
	EXTRN	DosCopy:FAR
	EXTRN	DosWrite:FAR
;
	SUBTTL	Data areas
	PAGE+
;
DGROUP	GROUP	_DATA
;
_DATA	SEGMENT	WORD PUBLIC 'DATA'
;
; Device driver header
;
HEADER	DD	-1			; Link to next device driver
	DW	8080H			; Device attributes:
					;   function level 001
					;   character device
	DW	OFFSET STRATEGY		; Strategy entry point
	DW	0			; IDC entry point - not used
	DB	'INIBAK$$'		; Device name
	DB	8 DUP (0)		; Reserved
;
; Filenames
;
SRCA	DB	'C:\OS2\OS2.INI',0	; First file to back up
	DB	49 DUP (0)		; Padding for patches
DSTA	DB	'C:\OS2\OS2.INB',0
	DB	49 DUP (0)		; Padding for patches
;
SRCB	DB	'C:\OS2\OS2SYS.INI',0	; Second file to back up
	DB	46 DUP (0)		; Padding for patches
DSTB	DB	'C:\OS2\OS2SYS.INB',0
	DB	46 DUP (0)		; Padding for patches
;
WLEN	DW	?			; Receives DosWrite length
;
CMES1	DB	'File ',0
CMES2	DB	' copied to ',0
CMES3	DB	CR,LF,0
;
_DATA	ENDS
;
	SUBTTL	Main code
	PAGE+
;
_TEXT	SEGMENT	WORD PUBLIC 'CODE'
;
	ASSUME	CS:_TEXT,DS:DGROUP,ES:NOTHING
;
; Strategy entry point; ES:BX points to the request packet
;
STRATEGY	PROC	FAR
;
	XOR	AX,AX			; set up to clear...
	CMP	ES:[BX+02H],AL		; initialise function?
	JE	STRA10			; j if so
	MOV	AX,8103H		; error and done status, unknown command
	JMP	SHORT STRA20		; use common exit code
;
STRA10:	CALL	INIT			; do the initialisation
;
STRA20:	MOV	ES:[BX+03H],AX		; store status in request packet
	RET				; return to system
;
STRATEGY	ENDP
;
	SUBTTL	Initialisation code
	PAGE+
;
; Initialisation code. All of this code is present only during initialisation;
; none of the driver data is used after that time either.
;
INIT	PROC	NEAR
;
	MOV	AX,OFFSET DGROUP:SRCA	; first file
	MOV	DX,OFFSET DGROUP:DSTA	; first backup
	CALL	DOCOPY			; copy it
;
	MOV	AX,OFFSET DGROUP:SRCB	; second file
	MOV	DX,OFFSET DGROUP:DSTB	; second backup
	CALL	DOCOPY			; copy it
;
	MOV	WORD PTR ES:[BX+0EH],OFFSET _TEXT:INIT
					; truncate code segment
	MOV	WORD PTR ES:[BX+10H],0	; lose data segment
	MOV	AX,0100H		; done status
	RET
;
INIT	ENDP
;
	SUBTTL	Copy a file
	PAGE+
;
; This routine copies a file
;
; Inputs:
;	AX	- offset of source file name
;	DX	- offset of destination file name
;
; Outputs:
;	None
;
DOCOPY	PROC	NEAR
;
	PUSH	DX			; save destination offset
	PUSH	AX			; save source offset
	PUSH	DS			; source segment
	PUSH	AX			; source offset
	PUSH	DS			; destination segment
	PUSH	DX			; destination offset
	PUSH	DCPY_EXISTING		; options
	PUSH	0			; DWORD reserved
	PUSH	0
	CALL	DosCopy			; do the copy
	OR	AX,AX			; copied OK?
	JNZ	NOCOPY			; j if not
	MOV	AX,OFFSET CMES1		; 'File '
	CALL	DOSOUT			; output it
	POP	AX			; source filename
	CALL	DOSOUT			; output it
	MOV	AX,OFFSET CMES2		; ' copied to '
	CALL	DOSOUT			; output it
	POP	AX			; destination filename
	CALL	DOSOUT			; output it
	MOV	AX,OFFSET CMES3		; '<CR><LF>'
	CALL	DOSOUT			; output it
;
NOCOPY:	RET				; and return
;
DOCOPY	ENDP
;
	SUBTTL	Output message
	PAGE+
;
; Routine to output a string to the screen.
;
; Inputs:
;	AX	- offset of zero terminated message
;
; Outputs:
;	None
;
DOSOUT	PROC	NEAR
;
	PUSH	ES			; save ES
	PUSH	AX			; save message offset
	PUSH	DS			; copy DS...
	POP	ES			; ...to ES
	MOV	DI,AX			; ES:DI point to message
	XOR	AL,AL			; set AL=0 for scan value
	MOV	CX,100			; just a large value
	REPNZ	SCASB			; scan for zero byte
	POP	AX			; recover message offset
	POP	ES			; recover ES
	SUB	DI,AX			; get size to DI
	DEC	DI			; adjust
	PUSH	STDOUT			; standard output handle
	PUSH	DS			; segment of message
	PUSH	AX			; offset of message
	PUSH	DI			; length of message
	PUSH	DS			; segment for length written
	PUSH	OFFSET DGROUP:WLEN	; offset for length written
	CALL	DosWrite		; write message
;
	RET
;
DOSOUT	ENDP
;
_TEXT	ENDS
;
	END
